module Moist
  class CampaignSubscriber < ApplicationRecord
    self.table_name = 'moist_campaign_subscribers'

    include ::Moist::Models::CampaignSubscriber

    has_many :moist_receipts, class_name: '::Moist::Receipt'
    has_many :moist_mailings, class_name: '::Moist::Mailing', foreign_key: :moist_campaign_subscriber_id
    has_one :next_moist_mailing, -> { unsent.order('send_at asc') }, class_name: '::Moist::Mailing', foreign_key: :moist_campaign_subscriber_id
    has_one :most_recent_moist_mailing, -> { sent.order('sent_at desc') }, class_name: '::Moist::Mailing', foreign_key: :moist_campaign_subscriber_id
    belongs_to :moist_campaign, class_name: '::Moist::Campaign'
    belongs_to :subscriber, polymorphic: true
    belongs_to :user

    after_create_commit :create_mailings!

    def create_mailings!
      moist_campaign.steps.each do |step|
        ::Moist::Mailing.create!(moist_campaign_subscriber: self, send_at: step.options[:delay].from_now, mailer_class: step.mailer_class, mailer_action: step.mailer_action)
      end
    end
  end
end
