module Moist
  module Models
    module Campaign
      def self.included(klass)
        klass.extend ClassMethods
      end

      def steps
        Steps::Collection.for(self)
      end

      def to_slug
        slug.to_sym
      end

      def subscribed?(subscriber, **args)
        moist_campaign_subscribers.exists?(subscriber: subscriber, **args)
      end

      module ClassMethods
        def [](val)
          campaign = ::Moist::Campaign.find_by(slug: val)
          raise ::Moist::UnknownCampaign, "Unknown campaign slug #{val}" if campaign.nil?

          campaign
        end

        def subscribe(subscriber, user:)
          SubscriptionManager.new(subscriber, user: user, action: :subscribe)
        end

        def unsubscribe(subscriber, user:)
          SubscriptionManager.new(subscriber, user: user, action: :unsubscribe)
        end
      end
    end
  end
end
