module Moist
  module Steps
    class Collection
      include Enumerable

      def self.for(campaign)
        new(campaign, ::Moist.step_registry[campaign.to_slug])
      end

      attr_reader :steps
      def initialize(campaign, steps)
        @campaign = campaign
        @steps = sort(steps)
      end

      def each(&block)
        @steps.each { |step| block.call(step) }
      end

      def size
        @steps.size
      end

      def [](val)
        @steps[val]
      end

      def find(step_number)
        return nil unless step_number.is_a?(Integer)

        steps.detect { |step| step.options[:step] == step_number } || find_next(step_number)
      end

      def find_next(step_number)
        return nil unless step_number.is_a?(Integer)

        steps.detect { |step| step.options[:step] > step_number } || last_step?(step_number)
      end

      private

      def last_step?(step_number)
        raise(::Moist::CampaignComplete) if steps.last.options[:step] == step_number

        nil
      end

      def sort(steps)
        steps.sort_by { |step| step.options[:step] }
      end
    end
  end
end
